@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2022 by the Free Software Foundation, Inc.
 #
 # This file is part of mailmanclient.
 #
@@ -16,10 +16,9 @@
 
 """Client code."""
 
-from __future__ import absolute_import, unicode_literals
-
 import warnings
 from operator import itemgetter
+from urllib.parse import quote
 
 from mailmanclient.constants import (MISSING)
 from mailmanclient.restobjects.address import Address
@@ -50,20 +49,51 @@ class Client:
     """Access the Mailman REST API root.
 
     :param baseurl: The base url to access the Mailman 3 REST API.
+    :type baseurl: str
     :param name: The Basic Auth user name.  If given, the `password` must
         also be given.
+    :type name: str
     :param password: The Basic Auth password.  If given the `name` must
         also be given.
+    :type password: str
+    :param request_hooks: Callable hooks to process request parameters before
+        being sent to Core's API.
+    :type request_hooks: List[callables]
     """
 
-    def __init__(self, baseurl, name=None, password=None):
+    def __init__(self, baseurl, name=None, password=None, request_hooks=None):
         """Initialize client access to the REST API."""
-        self._connection = Connection(baseurl, name, password)
+        self._connection = Connection(baseurl, name, password, request_hooks)
 
     def __repr__(self):
         return '<Client ({0.name}:{0.password}) {0.baseurl}>'.format(
             self._connection)
 
+    def add_hooks(self, request_hooks):
+        """Add a hook to process connections to Mailman's API.
+
+        Hooks are callables that are passed in the parameters of HTTP call made
+        to Mailman Core' RESt API and are expected to return the same number of
+        parameters back. This can be useful to manipulate parameters and update
+        them, or simply log each call to API.
+        ::
+
+            from mailmanclient.client import Client
+            client = Client('http://localhost:8001/3.1', <user>, <password>)
+
+            def sample_hook(params):
+                print(f'Request params are {params}')
+                return params
+
+            client.add_hook([sample_hook])
+
+
+        :param request_hooks: A list of callables that take in a dictionary of
+           parameters. ``params`` is the list of request parameters before the
+           request is made.
+        """
+        self._connection.add_hooks(request_hooks=request_hooks)
+
     @property
     def system(self):
         """Get the basic system information.
@@ -163,15 +193,19 @@ class Client:
         return [MailingList(self._connection, entry['self_link'], entry)
                 for entry in content['entries']]
 
-    def get_list_page(self, count=50, page=1, advertised=None):
+    def get_list_page(self, count=50, page=1, advertised=None, mail_host=None):
         """Get a list of all MailingList with pagination.
 
         :param count: Number of entries per-page (defaults to 50).
         :param page: The page number to return (defaults to 1).
         :param advertised: If marked True, returns all MailingLists including
                            the ones that aren't advertised.
+        :param mail_host: Domain to filter results by.
         """
-        url = 'lists'
+        if mail_host:
+            url = 'domains/{0}/lists'.format(mail_host)
+        else:
+            url = 'lists'
         if advertised:
             url += '?advertised=true'
         return Page(self._connection, url, MailingList, count, page)
@@ -214,6 +248,17 @@ class Client:
         """
         return self.get_list(fqdn_listname).get_member(subscriber_address)
 
+    def get_nonmember(self, fqdn_listname, nonmember_address):
+        """Get the Member object for a given MailingList and Non-member's
+        Email.
+
+        :param str fqdn_listname: Fully qualified address for the MailingList.
+        :param str subscriber_address: Email Address for the non-member.
+        :returns: A member of a list.
+        :rtype: :class:`Member`
+        """
+        return self.get_list(fqdn_listname).get_nonmember(nonmember_address)
+
     def get_member_page(self, count=50, page=1):
         """Return a paginated list of Members.
 
@@ -274,7 +319,7 @@ class Client:
         if alias_domain is not None:
             data['alias_domain'] = alias_domain
         response, content = self._connection.call('domains', data)
-        return Domain(self._connection, response['location'])
+        return Domain(self._connection, response.headers.get('location'))
 
     def delete_domain(self, mail_host):
         """Delete a Domain.
@@ -308,7 +353,7 @@ class Client:
             'users', dict(email=email,
                           password=password,
                           display_name=display_name))
-        return User(self._connection, response['location'])
+        return User(self._connection, response.headers.get('location'))
 
     def get_user(self, address):
         """Given an Email Address, return the User it belongs to.
@@ -401,16 +446,20 @@ class Client:
             data['password'] = password
         return self._connection.call('uris', data, 'PATCH')[1]
 
-    def find_lists(self, subscriber, role=None, count=50, page=1):
-        """Given a subscriber and a role, return all the list they are subscribed
-        to with given role.
+    def find_lists(self, subscriber, role=None, count=50, page=1,
+                   mail_host=None):
+        """Given a subscriber and a role, return all the list they are
+        subscribed to with given role.
 
         If no role is specified all the related mailing lists are returned
         without duplicates, even though there can potentially be multiple
         memberships of a user in a single mailing list.
 
         :param str subscriber: The address of the subscriber.
-        :param str role: owner, moderator or subscriber
+        :param str role: owner, moderator or subscriber.
+        :param int count: Number of entries per-page (defaults to 50).
+        :param int page: The page number to return (defaults to 1).
+        :param str mail_host: Domain to filter results by.
         :returns: A filtered list of mailing lists with given filters.
         :rtype: List[:class:`MailingList`]
         """
@@ -422,4 +471,31 @@ class Client:
         if 'entries' not in content:
             return []
         return [MailingList(self._connection, entry['self_link'], entry)
+                for entry in content['entries']
+                if not mail_host or entry['mail_host'] == mail_host]
+
+    def find_users(self, query):
+        """Find users with query string matching display name and emails.
+
+        :param str query: The string to search for. The search string is case
+            insensitive as core doens't care about the case.
+        :param int count: Number of entries per-page (defaults to 50).
+        :param int page: The page number to return (defaults to 1).
+        """
+        url = 'users/find?q={}'.format(quote(query))
+        response, content = self._connection.call(url)
+        if 'entries' not in content:
+            return []
+        return [User(self._connection, entry['self_link'], entry)
                 for entry in content['entries']]
+
+    def find_users_page(self, query, count, page):
+        """Same as :py:meth:`find_users` but allows for pagination.
+
+        :param str query: The string to search for. The search string is case
+            insensitive as core doens't care about the case.
+        :param int count: Number of entries per-page (defaults to 50).
+        :param int page: The page number to return (defaults to 1).
+        """
+        url = 'users/find?q={}'.format(quote(query))
+        return Page(self._connection, url, User, count, page)
